@@ -8,4 +8,4 @@ This program can be distributed under the terms of the GNU GPLv3.
 
 from __future__ import division, print_function, absolute_import
 
-__all__ = [ 'common', 'local', 's3', 's3s', 'gs', 'gss', 's3c', 's3cs', 'swift' ]
+__all__ = [ 'common', 'local', 's3', 'gs', 's3c', 'swift' ]
