@@ -10,7 +10,7 @@ from __future__ import division, print_function
 from _common import TestCase
 from contextlib import contextmanager
 from s3ql.backends import local
-from s3ql.backends.common import BucketPool, AbstractBucket
+from s3ql.backends.common import BackendPool, AbstractBackend
 from s3ql.block_cache import BlockCache
 from s3ql.mkfs import init_tables
 from s3ql.metadata import create_tables
@@ -29,8 +29,8 @@ class cache_tests(TestCase):
 
     def setUp(self):
 
-        self.bucket_dir = tempfile.mkdtemp()
-        self.bucket_pool = BucketPool(lambda: local.Bucket('local://' + self.bucket_dir, 
+        self.backend_dir = tempfile.mkdtemp()
+        self.backend_pool = BackendPool(lambda: local.Backend('local://' + self.backend_dir, 
                                                            None, None))
 
         self.cachedir = tempfile.mkdtemp()
@@ -49,7 +49,7 @@ class cache_tests(TestCase):
                          | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH,
                          os.getuid(), os.getgid(), time.time(), time.time(), time.time(), 1, 32))
 
-        self.cache = BlockCache(self.bucket_pool, self.db, self.cachedir + "/cache",
+        self.cache = BlockCache(self.backend_pool, self.db, self.cachedir + "/cache",
                                 self.max_obj_size * 100)
 
         # Tested methods assume that they are called from
@@ -57,10 +57,10 @@ class cache_tests(TestCase):
         llfuse.lock.acquire()
 
     def tearDown(self):
-        self.cache.bucket_pool = self.bucket_pool
+        self.cache.backend_pool = self.backend_pool
         self.cache.destroy()
         shutil.rmtree(self.cachedir)
-        shutil.rmtree(self.bucket_dir)
+        shutil.rmtree(self.backend_dir)
 
         llfuse.lock.release()
 
@@ -114,9 +114,9 @@ class cache_tests(TestCase):
 
         # We want to expire 4 entries, 2 of which are already flushed
         self.cache.max_entries = 16
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=2)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=2)
         self.cache.expire()
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
         self.assertEqual(len(self.cache.entries), 16)
 
         for i in range(20):
@@ -137,64 +137,64 @@ class cache_tests(TestCase):
         data3 = self.random_data(datalen)
 
         # Case 1: create new object
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=1)
         with self.cache.get(inode, blockno1) as fh:
             fh.seek(0)
             fh.write(data1)
             el1 = fh
         self.cache.upload(el1)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
         # Case 2: Link new object
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool)
         with self.cache.get(inode, blockno2) as fh:
             fh.seek(0)
             fh.write(data1)
             el2 = fh
         self.cache.upload(el2)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
         # Case 3: Upload old object, still has references
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=1)
         with self.cache.get(inode, blockno1) as fh:
             fh.seek(0)
             fh.write(data2)
         self.cache.upload(el1)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
         # Case 4: Upload old object, no references left
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_del=1, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_del=1, no_write=1)
         with self.cache.get(inode, blockno2) as fh:
             fh.seek(0)
             fh.write(data3)
         self.cache.upload(el2)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
         # Case 5: Link old object, no references left
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_del=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_del=1)
         with self.cache.get(inode, blockno2) as fh:
             fh.seek(0)
             fh.write(data2)
         self.cache.upload(el2)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
         # Case 6: Link old object, still has references
         # (Need to create another object first)
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=1)
         with self.cache.get(inode, blockno3) as fh:
             fh.seek(0)
             fh.write(data1)
             el3 = fh
         self.cache.upload(el3)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool)
         with self.cache.get(inode, blockno1) as fh:
             fh.seek(0)
             fh.write(data1)
         self.cache.upload(el1)
         self.cache.clear()
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
     def test_remove_referenced(self):
         inode = self.inode
@@ -203,7 +203,7 @@ class cache_tests(TestCase):
         blockno2 = 24
         data = self.random_data(datalen)
 
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=1)
         with self.cache.get(inode, blockno1) as fh:
             fh.seek(0)
             fh.write(data)
@@ -211,11 +211,11 @@ class cache_tests(TestCase):
             fh.seek(0)
             fh.write(data)
         self.cache.clear()
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool)
         self.cache.remove(inode, blockno1)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
     def test_remove_cache(self):
         inode = self.inode
@@ -238,12 +238,12 @@ class cache_tests(TestCase):
         with self.cache.get(inode, 1) as fh:
             fh.seek(0)
             fh.write(data1)
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=1)
         commit(self.cache, inode)
-        self.cache.bucket_pool.verify()
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_del=1)
+        self.cache.backend_pool.verify()
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_del=1)
         self.cache.remove(inode, 1)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
 
         with self.cache.get(inode, 1) as fh:
             fh.seek(0)
@@ -258,28 +258,28 @@ class cache_tests(TestCase):
         with self.cache.get(inode, 1) as fh:
             fh.seek(0)
             fh.write(data1)
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_write=1)
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_write=1)
         self.cache.clear()
-        self.cache.bucket_pool.verify()
-        self.cache.bucket_pool = TestBucketPool(self.bucket_pool, no_del=1)
+        self.cache.backend_pool.verify()
+        self.cache.backend_pool = TestBackendPool(self.backend_pool, no_del=1)
         self.cache.remove(inode, 1)
-        self.cache.bucket_pool.verify()
+        self.cache.backend_pool.verify()
         with self.cache.get(inode, 1) as fh:
             fh.seek(0)
             self.assertTrue(fh.read(42) == '')
 
-class TestBucketPool(AbstractBucket):
-    def __init__(self, bucket_pool, no_read=0, no_write=0, no_del=0):
-        super(TestBucketPool, self).__init__()
+class TestBackendPool(AbstractBackend):
+    def __init__(self, backend_pool, no_read=0, no_write=0, no_del=0):
+        super(TestBackendPool, self).__init__()
         self.no_read = no_read
         self.no_write = no_write
         self.no_del = no_del
-        self.bucket_pool = bucket_pool
-        self.bucket = bucket_pool.pop_conn()
+        self.backend_pool = backend_pool
+        self.backend = backend_pool.pop_conn()
         self.lock = threading.Lock()
 
     def __del__(self):
-        self.bucket_pool.push_conn(self.bucket)
+        self.backend_pool.push_conn(self.backend)
 
     def verify(self):
         if self.no_read != 0:
@@ -297,44 +297,44 @@ class TestBucketPool(AbstractBucket):
             yield self
 
     def lookup(self, key):
-        return self.bucket.lookup(key)
+        return self.backend.lookup(key)
 
     def open_read(self, key):
         self.no_read -= 1
         if self.no_read < 0:
             raise RuntimeError('Got too many open_read calls')
 
-        return self.bucket.open_read(key)
+        return self.backend.open_read(key)
 
     def open_write(self, key, metadata=None, is_compressed=False):
         self.no_write -= 1
         if self.no_write < 0:
             raise RuntimeError('Got too many open_write calls')
 
-        return self.bucket.open_write(key, metadata, is_compressed)
+        return self.backend.open_write(key, metadata, is_compressed)
 
     def is_temp_failure(self, exc):
-        return self.bucket.is_temp_failure(exc)
+        return self.backend.is_temp_failure(exc)
 
     def clear(self):
-        return self.bucket.clear()
+        return self.backend.clear()
 
     def contains(self, key):
-        return self.bucket.contains(key)
+        return self.backend.contains(key)
 
     def delete(self, key, force=False):
         self.no_del -= 1
         if self.no_del < 0:
             raise RuntimeError('Got too many delete calls')
 
-        return self.bucket.delete(key, force)
+        return self.backend.delete(key, force)
 
     def list(self, prefix=''):
-        '''List keys in bucket
+        '''List keys in backend
 
-        Returns an iterator over all keys in the bucket.
+        Returns an iterator over all keys in the backend.
         '''
-        return self.bucket.list(prefix)
+        return self.backend.list(prefix)
 
     def copy(self, src, dest):
         """Copy data stored under key `src` to key `dest`
@@ -342,7 +342,7 @@ class TestBucketPool(AbstractBucket):
         If `dest` already exists, it will be overwritten. The copying
         is done on the remote side. 
         """
-        return self.bucket.copy(src, dest)
+        return self.backend.copy(src, dest)
 
     def rename(self, src, dest):
         """Rename key `src` to `dest`
@@ -350,12 +350,12 @@ class TestBucketPool(AbstractBucket):
         If `dest` already exists, it will be overwritten. The rename
         is done on the remote side.
         """
-        return self.bucket.rename(src, dest)
+        return self.backend.rename(src, dest)
 
     def get_size(self, key):
         '''Return size of object stored under *key*'''
 
-        return self.bucket.get_size(key)
+        return self.backend.get_size(key)
 
 def commit(cache, inode, block=None):
     """Upload data for `inode`
